#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This file is part of rhythmbox-systray.
#
# Copyright (C) 2012-2013 - Elder Marco <eldermarco@gmail.com>
#
# rhythmbox-systray is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# rhythmbox-systray is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rhythmbox-systray. If not, see <http://www.gnu.org/licenses/>.
import os
import sys
import glob

import distutils.command.build
from distutils.core import setup
from distutils.cmd import Command

DATADIR = 'share'
LIBDIR = os.uname ()[-1] == 'x86_64' and  'lib64' or 'lib'


class build_extra (distutils.command.build.build):
    """
    Adds the extra commands to the build target. This class should be
    used with the core distutils. Adapted from python-distutils-extra
    """

    def __init__ (self, dist):
        distutils.command.build.build.__init__ (self, dist)
        self.user_options.extend ([('i18n', None, 'use the localisation')])

    def initialize_options (self):
        distutils.command.build.build.initialize_options (self)
        self.i18n = False

    def finalize_options (self):
        def has_i18n (command):
            return self.i18n == 'True' or                              \
                   ('build_i18n' in self.distribution.cmdclass and not \
                     self.i18n == 'False')

        distutils.command.build.build.finalize_options (self)
        self.sub_commands.append(('build_i18n', has_i18n))


class build (build_extra):
    """
    Adds the extra commands to the build target. This class should be
    used with setuptools. Adapted from python-distutil-extras
    """

    def finalize_options (self):
        def has_i18n(command):
            return self.i18n == 'True'

        distutils.command.build.build.finalize_options (self)
        self.sub_commands.append(('build_i18n', has_i18n))


class build_i18n (Command):
    """
    Class to build translation files (including .plugin.in files).
    Adapted from python-distutils-extra
    """
    description = "build .po and .plugin.in files"
    user_options = [('plugin-files=', None,
                     '.plugin.in file that should be merged'),
                    ('domain=', 'd', 'gettext domain'),
                    ('merge-po', 'm', 'merge po files against template'),
                    ('po-dir=', 'p', 'directory that holds the i18n files'),
                    ('bug-contact=', None, 'contact address for msgid bugs')]
    boolean_options = ['merge-po']

    def initialize_options (self):
        self.plugin_files = []
        self.domain = None
        self.merge_po = False
        self.po_dir = None
        self.bug_contact = None

    def finalize_options (self):
        if not self.po_dir:
            self.po_dir = 'po'
        if not self.domain:
            self.domain = self.distribution.metadata.name

    def run (self):
        """
        Update the language files, generate mo files and add them
        to the to be installed files
        """
        if not os.path.isdir (self.po_dir):
            return

        data_files = self.distribution.data_files
        if not data_files:
            # In case not data_files are defined in this script
            self.distribution.data_files = data_files = []

        if self.bug_contact:
            os.environ['XGETTEXT_ARGS'] = '--msgid-bugs-address=%s ' % \
                                          self.bug_contact

        # We want compile only languages listed in po/LINGUAS.
        linguas_file = os.path.join (self.po_dir, 'LINGUAS')
        if os.path.isfile (linguas_file):
            selected_languages = open (linguas_file).read ().split ()

        # Update po(t) files and print a report. We have to change the working
        # dir to the po dir for intltool
        wd = os.getcwd ()
        os.chdir (self.po_dir)
        cmd = ['intltool-update',
               (self.merge_po and '-r' or '-p'), '-g', self.domain]
        self.spawn (cmd)
        os.chdir (wd)
        max_po_mtime = 0
        for po_file in glob.glob ("%s/*.po" % self.po_dir):
            lang = os.path.basename (po_file[:-3])
            if selected_languages and not lang in selected_languages:
                continue

            mo_dir =  os.path.join ('build', 'mo', lang, 'LC_MESSAGES')
            mo_file = os.path.join (mo_dir, '%s.mo' % self.domain)
            if not os.path.exists (mo_dir):
                os.makedirs(mo_dir)

            cmd = ['msgfmt', po_file, '-o', mo_file]
            po_mtime = os.path.getmtime (po_file)
            mo_mtime = (os.path.exists (mo_file) and
                        os.path.getmtime (mo_file) or 0)
            if po_mtime > max_po_mtime:
                max_po_mtime = po_mtime
            if po_mtime > mo_mtime:
                self.spawn (cmd)

            targetpath = os.path.join ('share/locale', lang, 'LC_MESSAGES')
            data_files.append ((targetpath, (mo_file,)))

        # Merge .in with translation
        file_set = eval (self.plugin_files)
        for (target, files) in file_set:
            target = target.replace ('%libdir', LIBDIR)
            build_target = os.path.join ('build', target)
            if not os.path.exists (build_target):
               os.makedirs (build_target)

            files_merged = []
            for file in files:
                if file.endswith ('.in'):
                    file_merged = os.path.basename (file[:-3])
                else:
                    file_merged = os.path.basename(file)

                file_merged = os.path.join (build_target, file_merged)
                cmd = ['intltool-merge', self.po_dir, file, file_merged,
                       '-d', '-u', '-c',
                       '%s/.intltool-merge-cache' % self.po_dir]
                mtime_merged = os.path.exists (file_merged) and \
                               os.path.getmtime (file_merged) or 0
                mtime_file = os.path.getmtime (file)
                if mtime_merged < max_po_mtime or mtime_merged < mtime_file:
                    # Only build if output is older than input (.po,.in)
                    self.spawn (cmd)
                files_merged.append (file_merged)
            data_files.append ((target, files_merged))


setup (name            = 'rhythmbox-systray',
       version         = '1.0',
       description     = 'A Rhythmbox plugin to control the music player from '
                         'the system tray',
       author          = 'Elder Marco',
       author_email    = 'eldermarco@gmail.com',
       license         = 'GPLv3+',
       url             = 'http://github.com/elder/rhythmbox-systray',

       data_files      = [
           (LIBDIR + '/rhythmbox/plugins/systray/systray',
            glob.glob ('systray/*.py')),
           (DATADIR + '/rhythmbox/plugins/systray',
            glob.glob ('data/*.ui'))
       ],
       cmdclass        = { 'build'      :  build_extra,
                           'build_i18n' :  build_i18n }

       )
